@@ -82,6 +82,7 @@ struct _GSSDPClientPrivate {
         char              *server_id;
         char              *iface;
         char              *host_ip;
+        char              *network;
 
         GError            **error;
 
@@ -96,6 +97,7 @@ enum {
         PROP_MAIN_CONTEXT,
         PROP_SERVER_ID,
         PROP_IFACE,
+        PROP_NETWORK,
         PROP_HOST_IP,
         PROP_ACTIVE,
         PROP_ERROR
@@ -228,6 +230,10 @@ gssdp_client_get_property (GObject    *object,
                 g_value_set_string (value,
                                     gssdp_client_get_interface (client));
                 break;
+        case PROP_NETWORK:
+                g_value_set_string (value,
+                                    gssdp_client_get_network (client));
+                break;
         case PROP_HOST_IP:
                 g_value_set_string (value,
                                     gssdp_client_get_host_ip (client));
@@ -266,6 +272,9 @@ gssdp_client_set_property (GObject      *object,
         case PROP_IFACE:
                 client->priv->iface = g_value_dup_string (value);
                 break;
+        case PROP_NETWORK:
+                client->priv->network = g_value_dup_string (value);
+                break;
         case PROP_ACTIVE:
                 client->priv->active = g_value_get_boolean (value);
                 break;
@@ -315,6 +324,7 @@ gssdp_client_finalize (GObject *object)
         g_free (client->priv->server_id);
         g_free (client->priv->iface);
         g_free (client->priv->host_ip);
+        g_free (client->priv->network);
 
         G_OBJECT_CLASS (gssdp_client_parent_class)->finalize (object);
 }
@@ -406,6 +416,30 @@ gssdp_client_class_init (GSSDPClientClass *klass)
                           G_PARAM_STATIC_NICK |
                           G_PARAM_STATIC_BLURB));
 
+        /**
+         * GSSDPClient:network
+         *
+         * The network this client is currently connected to. You could set this
+         * to anything you want to identify the network this client is
+         * associated with. If you are using #GUPnPContextManager and associated
+         * interface is a WiFi interface, this property is set to the ESSID of
+         * the network. Otherwise, expect this to be the network IP address by
+         * default.
+         **/
+        g_object_class_install_property
+                (object_class,
+                 PROP_NETWORK,
+                 g_param_spec_string
+                         ("network",
+                          "Network ID",
+                          "The network this client is currently connected to.",
+                          NULL,
+                          G_PARAM_READWRITE |
+                          G_PARAM_CONSTRUCT |
+                          G_PARAM_STATIC_NAME |
+                          G_PARAM_STATIC_NICK |
+                          G_PARAM_STATIC_BLURB));
+
         /**
          * GSSDPClient:host-ip
          *
@@ -587,6 +621,46 @@ gssdp_client_get_host_ip (GSSDPClient *client)
         return client->priv->host_ip;
 }
 
+/**
+ * gssdp_client_set_network
+ * @client: A #GSSDPClient
+ * @network: The string identifying the network
+ *
+ * Sets the network identification of @client to @network.
+ **/
+void
+gssdp_client_set_network (GSSDPClient *client,
+                          const char  *network)
+{
+        g_return_if_fail (GSSDP_IS_CLIENT (client));
+
+        if (client->priv->network) {
+                g_free (client->priv->network);
+                client->priv->network = NULL;
+        }
+
+        if (network)
+                client->priv->network = g_strdup (network);
+
+        g_object_notify (G_OBJECT (client), "network");
+}
+
+/**
+ * gssdp_client_get_network
+ * @client: A #GSSDPClient
+ *
+ * Get the network this client is associated with.
+ *
+ * Return value: The network identification. This string should not be freed.
+ **/
+const char *
+gssdp_client_get_network (GSSDPClient *client)
+{
+        g_return_val_if_fail (GSSDP_IS_CLIENT (client), NULL);
+
+        return client->priv->network;
+}
+
 /**
  * gssdp_client_get_active
  * @client: A #GSSDPClient
@@ -778,12 +852,12 @@ socket_source_cb (GSSDPSocketSource *socket_source, GSSDPClient *client)
 {
         int type, len;
         char buf[BUF_SIZE], *end;
-        SoupMessageHeaders *headers;
+        SoupMessageHeaders *headers = NULL;
         GSocket *socket;
         GSocketAddress *address = NULL;
         gssize bytes;
         GInetAddress *inetaddr;
-        char *ip_string;
+        char *ip_string = NULL;
         guint16 port;
         GError *error = NULL;
         in_addr_t recv_network;
@@ -802,9 +876,8 @@ socket_source_cb (GSSDPSocketSource *socket_source, GSSDPClient *client)
         if (bytes == -1) {
                 g_warning ("Failed to receive from socket: %s",
                            error->message);
-                g_error_free (error);
 
-                return TRUE;
+                goto out;
         }
 
         /* We need the following lines to make sure the right client received
@@ -820,24 +893,22 @@ socket_source_cb (GSSDPSocketSource *socket_source, GSSDPClient *client)
                                          &error)) {
                 g_warning ("Could not convert address to native: %s",
                            error->message);
-                g_error_free (error);
 
-                return TRUE;
+                goto out;
         }
 
         recv_network = inet_netof (addr.sin_addr);
         our_addr.s_addr = inet_addr (gssdp_client_get_host_ip (client));
         our_network = inet_netof (our_addr);
         if (recv_network != our_network)
-                return TRUE;
+                goto out;
 
         if (bytes >= BUF_SIZE) {
-                g_object_unref (address);
                 g_warning ("Received packet of %u bytes, but the maximum "
                            "buffer size is %d. Packed dropped.",
                            (unsigned int) bytes, BUF_SIZE);
 
-                return TRUE;
+                goto out;
         }
 
         /* Add trailing \0 */
@@ -846,11 +917,10 @@ socket_source_cb (GSSDPSocketSource *socket_source, GSSDPClient *client)
         /* Find length */
         end = strstr (buf, "\r\n\r\n");
         if (!end) {
-                g_object_unref (address);
                 g_warning ("Received packet lacks \"\\r\\n\\r\\n\" sequence. "
                            "Packed dropped.");
 
-                return TRUE;
+                goto out;
         }
 
         len = end - buf + 2;
@@ -887,13 +957,18 @@ socket_source_cb (GSSDPSocketSource *socket_source, GSSDPClient *client)
                                headers);
         }
 
+out:
+        if (error)
+                g_error_free (error);
+
         if (ip_string)
                 g_free (ip_string);
 
         if (headers)
                 soup_message_headers_free (headers);
 
-        g_object_unref (address);
+        if (address)
+                g_object_unref (address);
 
         return TRUE;
 }
@@ -932,6 +1007,38 @@ is_primary_adapter (PIP_ADAPTER_ADDRESSES adapter)
         return !(adapter->IfType == IF_TYPE_SOFTWARE_LOOPBACK ||
                  family == AF_INET6);
 }
+
+static gboolean
+extract_address_and_prefix (PIP_ADAPTER_UNICAST_ADDRESS  adapter,
+                            PIP_ADAPTER_PREFIX           prefix,
+                            char                        *iface,
+                            char                        *network) {
+        DWORD ret = 0;
+        DWORD len = INET6_ADDRSTRLEN;
+
+        ret = WSAAddressToStringA (adapter->Address.lpSockaddr,
+                                   adapter->Address.iSockaddrLength,
+                                   NULL,
+                                   iface,
+                                   &len);
+        if (ret != 0)
+                return FALSE;
+
+        if (prefix) {
+                ret = WSAAddressToStringA (prefix->Address.lpSockaddr,
+                                           prefix->Address.iSockaddrLength,
+                                           NULL,
+                                           network,
+                                           &len);
+                if (ret != 0)
+                        return FALSE;
+        } else if (strcmp (iface, "127.0.0.1"))
+                strcpy (network, "127.0.0.0");
+        else
+                return FALSE;
+
+        return TRUE;
+}
 #endif
 
 /*
@@ -939,8 +1046,9 @@ is_primary_adapter (PIP_ADAPTER_ADDRESSES adapter)
  * it gets the IP of the first up & running interface and sets @interface
  * appropriately.
  */
+
 static char *
-get_host_ip (char **iface)
+get_host_ip (char **iface, char **network)
 {
 #ifdef G_OS_WIN32
         char *addr = NULL;
@@ -996,26 +1104,27 @@ get_host_ip (char **iface)
              ifaceptr != NULL;
              ifaceptr = ifaceptr->next) {
                 char ip[INET6_ADDRSTRLEN];
-                DWORD len = INET6_ADDRSTRLEN;
-                const char *p = NULL;
+                char prefix[INET6_ADDRSTRLEN];
+                const char *p, *q;
                 PIP_ADAPTER_ADDRESSES adapter;
-                SOCKET_ADDRESS unicast_address;
+                PIP_ADAPTER_UNICAST_ADDRESS address;
+
+                p = NULL;
 
                 adapter = (PIP_ADAPTER_ADDRESSES) ifaceptr->data;
-                unicast_address = adapter->FirstUnicastAddress->Address;
+                address = adapter->FirstUnicastAddress;
 
-                switch (unicast_address.lpSockaddr->sa_family) {
+                switch (address->Address.lpSockaddr->sa_family) {
                         case AF_INET:
                         case AF_INET6:
-                                ret = WSAAddressToStringA (
-                                        unicast_address.lpSockaddr,
-                                        unicast_address.iSockaddrLength,
-                                        NULL,
-                                        ip,
-                                        &len);
-                                if (ret == 0)
+                                if (extract_address_and_prefix (
+                                         address,
+                                         adapter->FirstPrefix,
+                                         ip,
+                                         prefix)) {
                                         p = ip;
-
+                                        q = prefix;
+                                }
                                 break;
                         default:
                                 continue;
@@ -1025,7 +1134,8 @@ get_host_ip (char **iface)
                         addr = g_strdup (p);
                         if (*iface == NULL)
                                 *iface = g_strdup (adapter->AdapterName);
-
+                        if (*network == NULL)
+                                *network = g_strdup (q);
                         break;
                 }
 
@@ -1072,9 +1182,11 @@ get_host_ip (char **iface)
              ifaceptr != NULL;
              ifaceptr = ifaceptr->next) {
                 char ip[INET6_ADDRSTRLEN];
-                const char *p;
-                struct sockaddr_in *s4;
-                struct sockaddr_in6 *s6;
+                char net[INET6_ADDRSTRLEN];
+                const char *p, *q;
+                struct sockaddr_in *s4, *s4_mask;
+                struct sockaddr_in6 *s6, *s6_mask;
+                struct in_addr net_addr;
 
                 p = NULL;
 
@@ -1085,11 +1197,20 @@ get_host_ip (char **iface)
                         s4 = (struct sockaddr_in *) ifa->ifa_addr;
                         p = inet_ntop (AF_INET,
                                        &s4->sin_addr, ip, sizeof (ip));
+                        s4_mask = (struct sockaddr_in *) ifa->ifa_netmask;
+                        net_addr.s_addr = (in_addr_t) s4->sin_addr.s_addr &
+                                          (in_addr_t) s4_mask->sin_addr.s_addr;
+                        q = inet_ntop (AF_INET, &net_addr, net, sizeof (net));
                         break;
                 case AF_INET6:
                         s6 = (struct sockaddr_in6 *) ifa->ifa_addr;
                         p = inet_ntop (AF_INET6,
                                        &s6->sin6_addr, ip, sizeof (ip));
+                        s6_mask = (struct sockaddr_in6 *) ifa->ifa_netmask;
+                        net_addr.s_addr =
+                                (in_addr_t) s6->sin6_addr.s6_addr &
+                                (in_addr_t) s6_mask->sin6_addr.s6_addr;
+                        q = inet_ntop (AF_INET6, &net_addr, net, sizeof (net));
                         break;
                 default:
                         continue; /* Unknown: ignore */
@@ -1100,6 +1221,8 @@ get_host_ip (char **iface)
 
                         if (*iface == NULL)
                                 *iface = g_strdup (ifa->ifa_name);
+                        if (*network == NULL)
+                                *network = g_strdup (q);
                         break;
                 }
         }
@@ -1118,7 +1241,8 @@ init_network_info (GSSDPClient *client)
 
         if (client->priv->iface == NULL || client->priv->host_ip == NULL)
                 client->priv->host_ip =
-                        get_host_ip (&client->priv->iface);
+                        get_host_ip (&client->priv->iface,
+                                     &client->priv->network);
 
         if (client->priv->iface == NULL) {
                 if (client->priv->error)
